
import unittest
from game import ConwaysGame
import numpy

class TestInit(unittest.TestCase):

    def test_init_empty(self):
        "__init__ should return empty board "
        game = ConwaysGame(shape=[10,10])
        self.assertEqual(game.live_cells_count(), 0)

    def test_init_data(self):
        "__init__ with params should copy the board"
        data = numpy.zeros(shape=[9,9])
        data[2][2] = data[1][2] = data[2][1] = True
        game = ConwaysGame(data=data)
        self.assertEqual(game.live_cells_count(), 3)

    def test_random_fill(self):
        "add_random_cells should add some cells"
        game = ConwaysGame(shape=[10,10])
        self.assertEqual(game.live_cells_count(), 0)
        game.add_random_cells()
        self.assertNotEqual(game.live_cells_count(), 0)

    def test_parametrized_fill(self):
        "add_random_cells(20) should add 20 cells"
        game = ConwaysGame(shape=[10,10])
        self.assertEqual(game.live_cells_count(), 0)
        game.add_random_cells(count=20)
        self.assertEqual(game.live_cells_count(), 20)

class TestSteps(unittest.TestCase):
    def test_pulsar(self):
        "Test pulsar"
        data = numpy.zeros(shape=[9,9])
        data[1][2] = data[2][2] = data[3][2] = True
        game = ConwaysGame(data=data)
        self.assertEqual(game.live_cells_count(), 3)
        game.next_step()
        self.assertEqual(game.live_cells_count(), 3)

    def test_glider(self):
        "Test glider moves"
        data = numpy.zeros(shape=[10,10])
        data[5][4] = data[6][4] = data[7][4] = True
        data[7][5] = data[6][6] = True
        game = ConwaysGame(data=data)
        for i in range(3):
            self.assertEqual(game.live_cells_count(), 5)
            game.next_step()
        for i in range(30):
            game.next_step()
        self.assertNotEqual(game.live_cells_count(), 5)

